/****************************************************************************


Dieses Programm soll Horst, den Kettenrobot mit den Getriebemotoren, bewegen.
Mittels eines Schwenkarms, der durch einen Servommotor realisiert wird,
wird ein Entfernungssensor bewegt.
Dadurch reduziert sich deren Anzahl von 3 auf 1.

Die Pins sind folgendermaen belegt:
PC0		Eingang fr das analoge Signal eines Sensors
PC1		mglicher Eingang eines zweiten Analogsensors
PC2		mglicher Eingang eines dritten Analogsensors
PB1		PWM(OCRA)-Ausgang fr Servo1 -> 1-2EN (L293D)
PB2		PWM(OCRB)-Ausgang fr Servo2 -> 3-4EN (L293D)
PB3		etwas anderer PWM(OC2)-Ausgang fr "Schwenkservo"
PD0		Ausgang fr Status-LED
PD1		1A (L293D)
PD2		2A (L293D)
PD3		4A (L293D)
PD4		3A (L293D)


Dies ist die von Steffen von der Mikrocontrollerspielwiese verbesserte und 
mit dem kleinen Servo funktionierende Variante

****************************************************************************/

#include <avr/io.h>    
#include <inttypes.h>
#define F_CPU 1000000UL  // 1 MHz
#include <util/delay.h>
#include <stdint.h>
#include <avr/interrupt.h>

#define SERVOMITTE 96 	//Mittlere Position des Servos 
#define SERVORECHTS 140 //Rechte Position des Servos
#define SERVOLINKS 53	//Linke Position des Servos

/****************************************************************************/
//Variablendeklarationen
int sample, i, value, a=SERVORECHTS;

/****************************************************************************/
//Zeitschschleife
void delay_ms(unsigned int ms)
{
	while(ms>0)
	{
		ms--;
		_delay_ms(0.96);
	}
}

//Fahrfunktionen, vorerst ohne PWM, d.h. Enable bekommt H oder L und keine Pulse
void vorwaerts()
	{
	PORTB=_BV(PB1) | _BV(PB2);
	PORTD=_BV(PD2) | _BV(PD3) | _BV(PD0);
	}
	
void rueckwaerts()
	{
	PORTB=_BV(PB1) | _BV(PB2);
	PORTD=_BV(PD1) | _BV(PD4);
	}
	
void stop()
	{
	PORTB=_BV(PB1) | _BV(PB2);
	delay_ms(100);				
	}
	
void linksdrehen()
	{
	PORTB=_BV(PB1) | _BV(PB2);
	PORTD=_BV(PD1) | _BV(PD3);
	}
	
void rechtsdrehen()
	{
	PORTB=_BV(PB1) | _BV(PB2);
	PORTD=_BV(PD2) | _BV(PD4);
	}
	
void messwerthandling()
	{
		sample=0;
		for(i=0;i<32;i++)    // Schleife zum Sammeln von 32 Messwerten  
		{
			ADCSRA |=_BV(ADSC);  // single conversion mode ein
      
			while ( ADCSRA & (1<<ADSC) ) {;}     // auf Abschluss der Konvertierung warten 
  
			sample+=ADCW;  // Aufsummierung der Samplewerte
		}
		
  		value=sample/32; // Aritmethisches Mittel der Samplewerte  
		
		//	value kann Werte zwischen 0(0V) und 1023(ca.5V) annehmen
		//	Der Analogsensor liefert Referenzspannungen von 0 (80cm) - 2.6V (10cm)
		// 	Daher ist die Grenze fr einen sinnvollen Schwellenwert (value) bei ca. 500!
		
		// Auswertung der Messdaten
		if (value <=300) //300 wrde etwa einer Entfernung von 20 cm entsprechen
		{
			vorwaerts();
			delay_ms(500);	// je nach dem, wie vorsichtig der Roboter fahren soll, kann man den Wert erhhen (schneller und riskanter)
							// oder verkleinern (langsamer und vorsichtiger)
		}
		
		else //es gab ein Hindernis
		{
			PORTD=0;
			stop();
			rueckwaerts();
			delay_ms(1000);
			stop();
			//Reagieren auf Position des Hindernisses
			if (a<100)			// Hindernis bei rechter Position des Servos
			{
				rechtsdrehen();
				delay_ms(1700);
			}
			else if (a==100)	// Hindernis in mittlerer Position
			{
				int randomd = rand();
				if(randomd%2 == 0)
				{linksdrehen();}
				else 
				{rechtsdrehen();}
				delay_ms(5000);
			}
			else if (a>100)		// Hindernis in linker Position
			{
				linksdrehen();
				delay_ms(1700);
			}
			stop();
		}	
	}

/****************************************************************************/
int main(void){ 

//Initialisierung der Datenrichtungsregister
  	DDRC=0; //PORTC als Eingang
  	PORTC=0;
  	DDRB=255; DDRD=255;    // PORTB & PORTD als Ausgabe
//Initialisierung der ADC-Register 
	ADMUX=0x00;  // ADC Ref auf Avcc, PC0 gewhlt, normale Formatierung
	ADCSRA=0x80; // ADC eingeschaltet, kein Prescale
//Initialisierung des OC2-PWMs
	TCCR2 = _BV(WGM20)     // Timer 2 Phase Correct PWM zhlt aufwrts bis Compare Match, danach abwrts (bis 0)
     	//| _BV(WGM21)     // wenn dieses Bit gesetzt wird, zhlt der Timer nur aufwrts
		| _BV(COM21)	   // "clear OC2 on compare match, set OC2 at TOP"
		| _BV(CS21);	   // Kein Prescale

/****************************************************************************/  
	int v_sensor=20; //Geschwindigkeit der Sensorbewegung: kleines v => schnelle Bewegung
	while(1)
	{
		//Bewegung des Servos
		while(a>=SERVOMITTE)//Rechts>>Mitte
		{
			OCR2=a;
			delay_ms(v_sensor);
			a--;
		}
		messwerthandling();
		
		while(a>=SERVOLINKS)//Mitte>>Links
		{	
			OCR2=a;
			delay_ms(v_sensor);
			a--;
		}
		messwerthandling();
		
		while(a<=SERVOMITTE)//Links>>Mitte
		{
			OCR2=a;
			delay_ms(v_sensor);
			a++;
		}
		messwerthandling();
		
		while(a<=SERVORECHTS)//Mitte>>Rechts
		{
			OCR2 = a;
			delay_ms(v_sensor);
			a++;
		}
		messwerthandling();
		
	}
	return 0;
  
}

